/**
 * Copyright 2013 Peergreen S.A.S.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ow2.util.scan.api.configurator.basic;

import java.lang.annotation.ElementType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.annotation.VisitorTarget;
import org.ow2.util.scan.api.annotation.VisitorType;
import org.ow2.util.scan.api.configurator.IAnnotationConfigurator;
import org.ow2.util.scan.api.metadata.structures.IAnnotation;

public class AnnotationConfigurator implements IAnnotationConfigurator {

    /**
     * List of registered visitors.
     */
    private final List<IAnnotationVisitor<?>> allAnnotationVisitors;

    /**
     * Constructor.
     */
    public AnnotationConfigurator() {
        allAnnotationVisitors = new ArrayList<>();
    }

    /**

    /**
     * Add visitor for the class
     * @param annotationVisitors the visitors to be launched on the class
     */
    @Override
    public void registerAnnotationVisitor(IAnnotationVisitor<?> annotationVisitor) {
        if (allAnnotationVisitors.contains(annotationVisitor)) {
            throw new IllegalArgumentException("the visitor " + annotationVisitor + "' already exists");
        }
        this.allAnnotationVisitors.add(annotationVisitor);
    }

    /**
     * Add visitors for the class
     * @param annotationVisitors the visitors to be launched on the class
     */
    @Override
    public void registerAnnotationVisitors(Collection<IAnnotationVisitor<?>> annotationVisitors) {
        this.allAnnotationVisitors.addAll(annotationVisitors);
    }


    /**
     * @param annotation signature of annotation
     * @return the visitor in the map or empty visitor
     */
    @Override
    public IAnnotationVisitor<?> getAnnotationVisitor(final IAnnotation annotation, ElementType elementType) {
        return filter(elementType).get(annotation.getInternalClassname());
    }

    /**
     * Filter all the given annotations by selecting the name for which they're registered and the type
     * @param elementType the given element type
     * @return
     */
    @Override
    public Map<String, IAnnotationVisitor<?>> filter(ElementType elementType) {
        Map<String, IAnnotationVisitor<?>> filteredMap = new HashMap<>();
        for (IAnnotationVisitor<?> visitor : allAnnotationVisitors) {
            Class<?> visitorClass = visitor.getClass();

            // get annotation type
            VisitorType visitorType = visitorClass.getAnnotation(VisitorType.class);
            VisitorTarget visitorTarget = visitor.getClass().getAnnotation(VisitorTarget.class);
            if (visitorTarget != null && visitorType != null) {
                if (Arrays.asList(visitorTarget.value()).contains(elementType)) {

                    // needs to encode the visitor type
                    String visitorTypeValue = visitorType.value();

                    // expected format is Lmy/example/MyClass;
                    visitorTypeValue = "L".concat(visitorTypeValue.replace(".", "/")).concat(";");
                    filteredMap.put(visitorTypeValue, visitor);
                }
            }
        }
        return filteredMap;
    }

}
