/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.api.configurator.basic;

import java.util.Arrays;

import org.ow2.util.scan.api.IScannerResult;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;

/**
 * ClassMetadataConfigurator is a configurator of class specific to ClassMetata.
 * @author Gael Lalire
 */
public class ClassMetadataConfigurator extends BasicConfigurator implements IClassConfigurator {

    /**
     * The class metadata.
     */
    private final IClassMetadata classMetadata;

    /**
     * @return the class metadata.
     */
    protected IClassMetadata getClassMetadata() {
        return classMetadata;
    }

    /**
     * Constructor.
     * @param classMetadata a class metadata
     */
    public ClassMetadataConfigurator(final IClassMetadata classMetadata) {
        this.classMetadata = classMetadata;
    }

    /**
     * Provide a method used by ClassConfigurator.
     * @param methodMetadata metadata of method
     * @return the configurator
     */
    @Override
    public IMethodConfigurator createMethodConfigurator(final IMethodMetadata methodMetadata) {
        MethodMetadataConfigurator methodMetadataConfigurator = new MethodMetadataConfigurator(methodMetadata);
        return methodMetadataConfigurator;
    }

    /**
     * Provide a method used by ClassConfigurator.
     * @param jField field signature
     * @return the configurator
     */
    @Override
    public final IFieldConfigurator createFieldConfigurator(final IFieldMetadata fieldMetadata) {
        FieldMetadataConfigurator fieldMetadataConfigurator = new FieldMetadataConfigurator(fieldMetadata);
        return fieldMetadataConfigurator;
    }



    /**
     * call configuration complete.
     * @param referencedClasses you may fill this set with class referenced in current metadata
     */
    @Override
    public void configurationComplete(IScannerResult scannerResult) {
        String[] interfaces = classMetadata.getJClass().getInterfaces();
        if (interfaces != null) {
            scannerResult.getReferencedClasses().addAll(Arrays.asList(interfaces));
        }
        String superName = classMetadata.getJClass().getSuperName();
        if (!superName.equals("java/lang/Object")) {
            scannerResult.getReferencedClasses().add(superName);
        }
        super.configurationComplete(scannerResult);
    }


}
