/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl.metadata;

import org.ow2.util.scan.api.metadata.structures.IField;

/**
 * This class defines a Field object. It is not based on reflection.
 * @author Florent Benoit
 */
public final class JField extends JMember implements IField {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 1872275434794410559L;

    /**
     * Field's descriptor.
     */
    private String descriptor = null;

    /**
     * Value of the field.
     */
    private final Object value;

    /**
     * Constructor. *
     * @param access the field's access flags (see
     *        {@link org.ow2.easybeans.asm.Opcodes}). This parameter also indicates
     *        if the field is synthetic and/or deprecated.
     * @param name the field's name.
     * @param descriptor the field's descriptor (see
     *        {@link org.ow2.util.scan.api.IType}).
     * @param signature the field's signature. May be <tt>null</tt> if the
     *        field's type does not use generic types.
     * @param value the field's initial value. This parameter, which may be
     *        <tt>null</tt> if the field does not have an initial value, must
     *        be an {@link Integer}, a {@link Float}, a {@link Long}, a
     *        {@link Double} or a {@link String} (for <tt>int</tt>,
     *        <tt>float</tt>, <tt>long</tt> or <tt>String</tt> fields
     *        respectively). <i>This parameter is only used for static fields</i>.
     *        Its value is ignored for non static fields, which must be
     *        initialized through bytecode instructions in constructors or
     *        methods.
     */
    public JField(final int access, final String name, final String descriptor, final String signature,
            final Object value) {
        super();
        setAccess(access);
        setName(name);
        this.descriptor = descriptor;
        setSignature(signature);
        Object tmpValue = value;
        if (tmpValue != null) {
            if (!(tmpValue instanceof Integer) && !(tmpValue instanceof Float) && !(tmpValue instanceof Long)
                    && !(tmpValue instanceof Double) && !(tmpValue instanceof String)) {
                throw new IllegalArgumentException("value should represent a primitive type");
            }
        }
        this.value = tmpValue;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param obj object to compare
     * @return true if given object is equals
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj != null && obj instanceof JField) {
            JField other = (JField) obj;

            // same name
            if (!this.getName().equals(other.getName())) {
                return false;
            }

            // same descriptor
            if ((this.descriptor != null) && (!this.descriptor.equals(other.descriptor))) {
                return false;
            }

            // same signature
            if ((this.getSignature() != null) && (!this.getSignature().equals(other.getSignature()))) {
                return false;
            }

            // if all tests succeed, return true
            return true;
        }
        return false;
    }

    /**
     * @return field's descriptor.
     */
    @Override
    public String getDescriptor() {
        return descriptor;
    }

    /**
     * @return field's value.
     */
    @Override
    public Object getValue() {
        return value;
    }


    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // name
        sb.append("[name=");
        sb.append(getName());

        // access
        sb.append(", access=");
        sb.append(getAccess());

        // descriptor
        if (descriptor != null) {
            sb.append(", descriptor=");
            sb.append(descriptor);
        }

        // signature
        if (getSignature() != null) {
            sb.append(", signature=");
            sb.append(getSignature());
        }

        // exceptions
        if (value != null) {
            sb.append(", value=");
            sb.append(value);
        }
        sb.append("]");
        return sb.toString();
    }

}
