/**
 * Copyright 2013 Peergreen S.A.S.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ow2.util.scan.impl;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

import java.net.URL;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.IScannerResult;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class TestScanner {

    private IScanner scanner;

   private TestSessionConfigurator sessionConfigurator;

   private IScannerResult result;

   private IClassMetadata classMetadata;

    @BeforeClass
    public void scan() throws ScanException {
        scanner = new ASMScannerImpl();

        URL url = TestScanner.class.getResource("/".concat(MyClassToScan.class.getName().replace(".", "/").concat(".class")));
        this.sessionConfigurator = new TestSessionConfigurator();
        sessionConfigurator.getTestAnnotationConfigurator().registerAnnotationVisitor(new MyDummyAnnotationVisitor());
        this.result = scanner.scanClass(url, sessionConfigurator);

        Map<String, IClassMetadata> classes = sessionConfigurator.getClasses();

        // Only one class
        assertEquals(classes.size(), 1);

        this.classMetadata = classes.get(MyClassToScan.class.getName());
    }


    @Test
    public void testClasses() {
        assertNotNull(classMetadata);
    }

    @Test()
    public void testFields() {

        // Test that fields are here
        Collection<IFieldMetadata> fields = classMetadata.getFieldMetadataCollection();
        assertEquals(fields.size(), 2);

    }


    @Test()
    public void testMethods() {

        // Test that methods are here
        List<IMethodMetadata> methods = classMetadata.searchMethodMetadata("dummyMethod");
        assertEquals(methods.size(), 1);
        assertEquals(methods.get(0).getJMethod().getDescriptor(), "()V");

        methods = classMetadata.searchMethodMetadata("mySetterMethod");
        assertEquals(methods.size(), 1);
        assertEquals(methods.get(0).getJMethod().getDescriptor(), "(Ljava/lang/String;)V");
    }

    @Test
    public void testReferencedClasses() {

        Collection<String> referencedClasses = result.getReferencedClasses();
        assertNotNull(referencedClasses);
        assertEquals(referencedClasses.size(), 1);
        assertTrue(referencedClasses.contains("java/lang/Cloneable"));
    }

    @Test
    public void testAnnotationProcessingOnClass() {

        // check if metadata has been added
        MyDummyAnnotationStruct annotationStruct = classMetadata.get(MyDummyAnnotationStruct.class);
        assertNotNull(annotationStruct);
        assertEquals(annotationStruct.getValue(), "myValueOnClass");
        assertEquals(annotationStruct.getTestValue(), "myTestValueOnClass");

    }


    @Test
    public void testAnnotationProcessingOnFields() {
        // check if metadata has been added
        Collection<IFieldMetadata> myFieldList = classMetadata.searchFieldMetadata("myField");
        assertNotNull(myFieldList);
        assertEquals(myFieldList.size(), 1);
        IFieldMetadata field = myFieldList.iterator().next();
        MyDummyAnnotationStruct annotationStruct = field.get(MyDummyAnnotationStruct.class);
        assertNotNull(annotationStruct);
        assertEquals(annotationStruct.getValue(), "myValueOnFields");
        assertEquals(annotationStruct.getTestValue(), "myTestValueOnField");

        //no annotation on the other field
        myFieldList = classMetadata.searchFieldMetadata("myOtherField");
        assertNotNull(myFieldList);
        assertEquals(myFieldList.size(), 1);
        field = myFieldList.iterator().next();
        annotationStruct = field.get(MyDummyAnnotationStruct.class);
        assertNull(annotationStruct);
    }


    @Test
    public void testAnnotationProcessingOnMethods() {
     // check if metadata has been added
        Collection<IMethodMetadata> myMethodList = classMetadata.searchMethodMetadata("dummyMethod");
        assertNotNull(myMethodList);
        assertEquals(myMethodList.size(), 1);
        IMethodMetadata method = myMethodList.iterator().next();
        MyDummyAnnotationStruct annotationStruct = method.get(MyDummyAnnotationStruct.class);
        assertNotNull(annotationStruct);
        assertEquals(annotationStruct.getValue(), "myValueOnMethod");
        assertEquals(annotationStruct.getTestValue(), "myTestValueOnMethod");

        //no annotation on the other field
        myMethodList = classMetadata.searchMethodMetadata("mySetterMethod");
        assertNotNull(myMethodList);
        assertEquals(myMethodList.size(), 1);
        method = myMethodList.iterator().next();
        annotationStruct = method.get(MyDummyAnnotationStruct.class);
        assertNull(annotationStruct);


    }


}
